classdef fastFmmTree < handle
    
    properties
        parent %
        children %
        numChildren %
        numNodes %
        %
        level %
        sisters
        numSisters
        %
        centres %
        index %
        coordinates %
        grid %
        xmin
        xmax
        ymin
        ymax
        centre
        width
        maxLevel
        removed
        dpointleaf
        first
    end
    
    methods
        
        %-----------------------------------------
        %
        %-----------------------------------------
        
        function self = fastFmmTree(maxLevel,points)
            
            %-----------------------------------------
            %
            %-----------------------------------------
            
            % store inputs
            self.maxLevel = maxLevel;
            
            %-----------------------------------------
            %
            %-----------------------------------------
            
            % work out how many nodes are at each level
            numNodesAtLevel = [4.^(0:maxLevel) length(points)];
            
            % work out the total number of nodes
            self.numNodes = sum(numNodesAtLevel);
            
            % array of pointers to the first node at each level
            self.first = [0 cumsum(numNodesAtLevel)]+1;
            
            %-----------------------------------------
            % allocate space for tree data
            %-----------------------------------------
            
            self.parent = zeros(self.numNodes,1);
            self.children = zeros(self.numNodes,20);
            self.numChildren = zeros(self.numNodes,1);
            self.level = zeros(self.numNodes,1);
            self.sisters = zeros(self.numNodes,20);
            self.numSisters = zeros(self.numNodes,1);
            self.centres = zeros(self.numNodes,1);
            self.coordinates = zeros(self.numNodes,2);
            self.index = zeros(self.numNodes,1);
            self.removed = zeros(self.numNodes,1);
            self.dpointleaf = zeros(length(points),1);
            
            %-----------------------------------------
            % calculate centres of boxes
            %-----------------------------------------
            
            self.xmin = min(real(points));
            self.xmax = max(real(points));
            self.ymin = min(imag(points));
            self.ymax = max(imag(points));
            
            self.width = max(self.xmax-self.xmin,self.ymax-self.ymin);
            self.centre = 0.5*(self.xmax+self.xmin) + 1i*0.5*(self.ymax+self.ymin);
            
            %-----------------------------------------
            % setup tree
            %-----------------------------------------
            
            % loop through levels
            for level = 0:self.maxLevel
                
                k = self.first(level+1);
                
                w = self.width/2^level;
                
                self.grid{level+1} = zeros(2^level,2^level);
                
                for j=1:2^level
                    for i=1:2^level
                        
                        self.coordinates(k,:) = [i j];
                        self.level(k) = level;
                        self.grid{level+1}(i,j) = k;
                        
                        self.centres(k) = self.xmin + (i-0.5)*w + ...
                            1i*(self.ymin + (j-0.5)*w);
                        
                        if self.level<self.maxLevel
                            
                            self.numChildren(k) = 4;
                            
                            kids = [
                                self.first(level+2) + 2*2^(level+1)*(j-1)+2*(i-1),
                                self.first(level+2) + 2*2^(level+1)*(j-1)+2*(i-1)+1,
                                self.first(level+2) + 2*2^(level+1)*(j-1)+2^(level+1)+2*(i-1),
                                self.first(level+2) + 2*2^(level+1)*(j-1)+2^(level+1)+2*(i-1)+1
                                ];
                            
                            self.children(k,1:4) = kids;
                            
                            self.parent(kids) = k;
                            
                        end
                        
                        k = k+1;
                        
                    end
                end
                
                
            end % level
            
            %-----------------------------------------
            % addpoints
            %-----------------------------------------
            
            child = self.first(maxLevel+2);
            
            for j=1:length(points)
                
                [parent,self.dpointleaf(j)] = self.getNearestLeaf(points(j));
                
                if self.numChildren(parent) >= size(self.children,2)
                    error('Maximum number of children of node %d is exceeded.',parent)
                end
                
                self.children(parent,self.numChildren(parent)+1) = child;
                self.numChildren(parent) = self.numChildren(parent)+1;
                self.parent(child) = parent;
                
                self.index(child) = j;
                self.level(child) = self.maxLevel + 1;
                
                % get sisters
                numSisters = self.numChildren(parent);
                sisters = self.children(parent,1:numSisters);
                
                self.centres(child) = points(j);
                
                % add other children as sisters
                self.sisters(child,1:numSisters) = sisters;
                self.numSisters(child) = numSisters;
                
                for k=1:length(sisters)
                    
                    % add self as sister to the other children
                    ns = self.numSisters(sisters(k));
                    self.numSisters(sisters(k)) = ns+1;
                    self.sisters(sisters(k),ns+1) = child;
                    
                end
                
                child = child+1;
                
            end
            
        end
        
        %-----------------------------------------
        %
        %-----------------------------------------
        
        function [index,dist] = getNearestLeaf(self,point)
            
            % start at head node
            index = 1;
            
            while self.level(index) < self.maxLevel
                
                if self.numChildren(index) == 0
                    error('Expected node %d to have children.',index)
                end
                
                % find which child is closer...
                
                % work out distances to children
                d = abs(point - self.centres(self.children(index,1:self.numChildren(index))));
                
                % find closest child
                [dist,j] = min(d);
                
                % move to the closest child
                index = self.children(index,j);
                
            end
            
        end
        
        %-----------------------------------------
        %
        %-----------------------------------------
        
        function trim(self)
            
            for level = self.maxLevel:-1:0
                
                for j = self.first(level+1):self.first(level+2)-1
                    
                    if isempty(self.getChildren(j))
                        
                        self.removed(j) = 1;
                        
                    end
                    
                end
                
            end
            
        end
        
        %-----------------------------------------
        %
        %-----------------------------------------
        
        function indices = getChildren(self,index)
            
            indices = self.children(index,1:self.numChildren(index));
            
            removed = self.removed(indices);
            
            indices = indices(~removed);
            
        end
        
        function [classA,classB] = getNear(self,index)
            
            [classA,classB] = getNear2(self,index,1);
            
        end
        
        function [classA,classB] = getNear2(self,index,order)
            
            if order==0
                error('order must be greater than 0')
            end
            
            classA = self.getSiblings(index);
            classB = [];
            
            
            if order>=1
                gramps = self.getParentX(index,order-1);
                if gramps~=0
                    uncles = self.getNeighbours(gramps);
                    classA = [classA,self.getChildrenX(uncles,order-1)];
                end
            end
            
            gramps = self.getParentX(index,order);
            if gramps~=0
                uncles = self.getNeighbours(gramps);
                classB = self.getChildrenX(uncles,order);
            end
            
            classB = setdiff(classB,classA);
            classA = setdiff(classA,[index]);
            
            removed = self.removed(classA);
            classA = classA(find(~removed));
            
            removed = self.removed(classB);
            classB = classB(find(~removed));
            
        end
        
        function list = getChildrenX(self,list,order)
            
            for k = 1:order
                
                list = getChildrenList(self,list);
                
            end
            
        end
        
        function node = getParentX(self,index,order)
            
            % get list version of getNeighbours
            % list version of get children
            
            node = index;
            
            k = 1;
            
            while node ~= 0 && k <= order
                
                node = self.parent(node);
                k = k+1;
                
            end
            
        end
        
        %-----------------------------------------
        %
        %-----------------------------------------
        
        function val = getNeighbours(self,index)
            
            i = self.coordinates(index,1);
            j = self.coordinates(index,2);
            
            L = self.getLevel(index);
            
            if L <= self.maxLevel
            
                patterni = [i-1,i,i+1,i+1,i+1,i,i-1,i-1];
                patternj = [j-1,j-1,j-1,j,j+1,j+1,j+1,j];
                
                maski = (patterni > 0) & (patterni <= 2^L);
                maskj = (patternj > 0) & (patternj <= 2^L);
                
                mask = find(maski & maskj);
                
                val = self.grid{L+1}(sub2ind(size(self.grid{L+1}),...
                    patterni(mask),patternj(mask)));
            
            else
                
                val = [];
            
            end
            
        end
        
        function val = getChildrenList(self,list)
            
            val = [];
            
            for k=1:length(list)
                val = [val,self.children(list(k),...
                    1:self.numChildren(list(k)))];
            end
            
        end
        
        %-----------------------------------------
        %
        %-----------------------------------------

        function val = getNodesByLevel(self,level)
        
            %val = find(self.level==level);
            
            val = self.first(level+1):self.first(level+2)-1;
            
        end
        
        %-----------------------------------------
        %
        %-----------------------------------------
        
        function indices = getSiblings(self,index)
            
            indices = self.sisters(index,1:self.numSisters(index));
            
        end
        
        %-----------------------------------------
        % 
        %-----------------------------------------

        function L = getLevel(self,index)
            
            L = self.level(index);
            
        end
        
    end % end methods
    
end