classdef hawkins_fmm < fastFmmTree
    
    properties
        radiating
        regular
        diffHH
        diffHJ
        diffJJ
        inputorder
        minLevel
        kwave
    end
    
    methods
        
        function self = hawkins_fmm(maxLevel,points,kwave)
            
            self = self@fastFmmTree(maxLevel,points)            
            
            self.diffHH = 10;
            self.diffHJ = 10;
            self.diffJJ = 10;

            self.kwave = kwave;
            
        end
        
        
        function upwardPassAction(self,index)
           
            %----------------------------------------------
            % transfer to siblings
            %----------------------------------------------
            
            [classA,classB] = self.getNear(index);

            if self.getLevel(index) > self.maxLevel
                                
                % then we need to broadcast to all siblings
                for j=1:length(classA)

                    self.regular{classA(j)} = self.regular{classA(j)} + ...
                        regularwavefunctionexpansion(self.radiating{index},...
                        self.centres(classA(j)),self.regular{classA(j)}.order);
                    
                end
                
            end
            
            %----------------------------------------------
            % transfer to cousins
            %----------------------------------------------
            
            % then we need to broadcast to all cousins
            for j=1:length(classB)
                
                self.regular{classB(j)} = self.regular{classB(j)} + ...
                    regularwavefunctionexpansion(self.radiating{index},...
                    self.centres(classB(j)),self.regular{classB(j)}.order);
                
            end
                
            %----------------------------------------------
            % transfer to parent
            %----------------------------------------------

            parent = self.parent(index);
            
            if parent~=0
                
                self.radiating{parent} = self.radiating{parent} + ...
                    radiatingwavefunctionexpansion(self.radiating{index},...
                    self.centres(parent));

            end
            
        end

        
        function downwardPassAction(self,index)
            
            children = self.getChildren(index);
            
            % then we need to broadcast to all cousins
            for j=1:length(children)
                
                self.regular{children(j)} = self.regular{children(j)} + ...
                    regularwavefunctionexpansion(self.regular{index},...
                    self.centres(children(j)));
                
            end
            
        end
        
        function importExpansion(self,vectorin,orderin)

            self.inputorder = orderin(:);
            
            nmax = max(self.inputorder);
            kwave = self.kwave;

            numscat = length(orderin);

            
            for j=1:self.numNodes
                self.regular{j} = regularzero(nmax+self.diffHJ,self.centres(j),kwave);
            end
            
            for j=1:self.numNodes
                self.radiating{j} = radiatingzero(nmax+self.diffHH,self.centres(j),kwave);
            end
            
            ii = find(self.index);
            jj = self.index(ii);
            
            kk = 1;
            for k=1:length(ii)
                cof = vectorin(kk:kk+2*self.inputorder(k));
                nn = (-self.inputorder(k):self.inputorder()).';
                cof = spdiags((-1).^((nn-abs(nn))/2),0,...
                    2*self.inputorder(k)+1,2*self.inputorder(k)+1) * cof;
                expansionin{jj(k)} = radiatingwavefunctionexpansion(self.inputorder(k), ...
                    self.centres(ii(k)),kwave,cof);
                kk = kk + 2*self.inputorder(k)+1;
            end                    
            
            for k=1:length(ii)
                self.radiating{ii(k)} = radiatingwavefunctionexpansion(...
                    expansionin{jj(k)},self.centres(ii(k)),nmax+self.diffHH);
            end
            
        end
        
        function val = exportExpansion(self,nmax)
           
            if isempty(self.inputorder)
                error('must use importExpansion first')
            end
            
            ii = find(self.index);
            jj = self.index(ii);

            %val(jj) = self.regular(ii);

            for k=1:length(ii)
                tmp{jj(k)} = regularwavefunctionexpansion(...
                    self.regular{ii(k)},self.centres(ii(k)),self.inputorder(jj(k)));
            end
            
            val = zeros(sum(self.inputorder),1);

            kk = 1;
            for k=1:length(ii)
                cof = tmp{jj(k)}.coefficients;
                nn = (-self.inputorder(k):self.inputorder()).';
                cof = spdiags((-1).^((nn-abs(nn))/2),0,...
                    2*self.inputorder(k)+1,2*self.inputorder(k)+1) * cof;
                val(kk:kk+2*self.inputorder(k)) = cof;
                kk = kk + 2*self.inputorder(k)+1;
            end                    

        end
        
        function upwardPass(self)
            
            % pass up as far as minlevel
            for level = self.maxLevel+1:-1:self.minLevel+1
                
                ii = self.getNodesByLevel(level);

                % find nodes that are not removed
                jj = find(~self.removed(ii));
                
                for k = 1:length(jj)
                    
                    self.upwardPassAction(ii(jj(k)));
                    
                end
                
            end
                
            % get all nodes at minlevel
            ii = self.getNodesByLevel(self.minLevel);

            % find nodes that are not removed
            jj = find(~self.removed(ii));
                
            % pass to all nodes apart from classA at the minlevel
            for j=1:length(jj)
                
                [classA,classB] = self.getNear(ii(jj(j)));
                
                temp = setdiff(ii,classA);
%                 temp = setdiff(temp,classB);
                temp = setdiff(temp,ii(jj(j)));
                
                reg = self.regular{temp};
                rad = self.radiating;
                centres = self.centres;
                
                for k = 1:length(temp)
%                     self.regular{temp(k)} = self.regular{temp(k)} + ii(j)*(self.radiating{ii(j)}~=0);
                    self.regular{temp(k)} = self.regular{temp(k)} + ...
                        regularwavefunctionexpansion(self.radiating{ii(jj(j))},...
                        centres(temp(k)),self.regular{temp(k)}.order);
               
                end
                
%                self.regular{temp} = reg;
                
            end
            
        end
                
        function downwardPass(self)

            %for level = self.maxLevel+1:-1:self.minLevel+1
           
            for level = self.minLevel:self.maxLevel+1
                
                ii = self.getNodesByLevel(level);

                jj = find(~self.removed(ii));
                
                for k = 1:length(jj)
                    
                    self.downwardPassAction(ii(jj(k)));
                    
                end
                
            end
                
        end
        
    end
    
end