function [pos, FSP, R_miz, conc] = FSD_gen

%% Geometry

% MIZ radius (m)
R_miz = 1000;

% ice thickness
thick = 1;

% concentration
conc = 0.7;

% Min and Max floe radii (m)
Rmin = 10; Rmax = 120;

% Lognormal distribution parameters
tau = 5;
mode = 20;
sigma = 0.6;
s = (mode-tau)*exp(sigma^2);
mu = log(s);

% Number of bins
Nbins = Rmax/10;

% Critical radius
Rad = linspace(Rmin,Rmax,Nbins);
[~,ind] = min(abs(Rad-mode));
Rcrit = Rad(ind);


% Floe size and number sampling routine in circular MIZ
[Nf, Rad, R_miz_mod] = FSD_lognormal_circ_MIZ(Rmin, Rmax, tau, mu, sigma, ...
    conc, Nbins, R_miz);

% Mean floe radius
Rmean = sum(Nf.*Rad)/sum(Nf);

% Remove small floes
[~,ii] = min(abs(Rad - Rcrit));
Nf = Nf(ii:end); Rad = Rad(ii:end);

% Remove empty bins
Rad = Rad(Nf>0);
Nf = Nf(Nf>0);
Nbins = length(Nf);

% Floe size pool containing properties of unique floes
FSP = [Rad.' thick*ones(Nbins,1)];

% Number of floes
Nfloes = sum(Nf);

% FSD generated with circle packing algo for circular MIZ
pos = circle_packing(Nf, Rad, R_miz_mod);
% disp('>>>>>>>>>>>>>>> Circular MIZ generated <<<<<<<<<<<<<<<<<')


%% Visualization
vis = 0;
if vis == 1
    visu_geom(pos,FSP(:,1))
end


%%%%%% Routines

function [Nf, Rad, Rmiz_adj] = FSD_lognormal_circ_MIZ(Rmin, Rmax, tau, ...
    mu, sigma, c, Nb, Rmiz)

% Nf = FSD_power_law(Dmin, Dmax, alpha, c, Nb, W, H)
%
% Binning procedure to generate a FSD based on a lognormal distribution.

% Bins of floe radii
Rad = linspace(Rmin, Rmax, Nb);

% Log variable
Rad_log = log(Rad - tau);

% Normalisation (in log(a) space)
loga = linspace(Rad_log(1),Rad_log(end),1001);
Norm = trapz(loga, 1/(sigma*sqrt(2*pi))*exp(-(loga-mu).^2/(2*sigma^2)));


% Proportion of floes in each bin (based on power law distribution)
prop = zeros(1,Nb);
loga = linspace(Rad_log(1),0.5*(Rad_log(1)+Rad_log(2)),101);
prop(1) = (1/Norm)*trapz(loga, 1/(sigma*sqrt(2*pi))*...
    exp(-(loga-mu).^2/(2*sigma^2)));
for i = 2:Nb-1
    loga = linspace(0.5*(Rad_log(i-1)+Rad_log(i)),...
        0.5*(Rad_log(i)+Rad_log(i+1)),101);
    prop(i) = (1/Norm)*trapz(loga, 1/(sigma*sqrt(2*pi))*...
    exp(-(loga-mu).^2/(2*sigma^2)));
end
loga = linspace(0.5*(Rad_log(end-1)+Rad_log(end)),Rad_log(end),101);
prop(end) = (1/Norm)*trapz(loga, 1/(sigma*sqrt(2*pi))*...
    exp(-(loga-mu).^2/(2*sigma^2)));

% Sanity check
if abs(sum(prop)-1) > 1e-4
    disp(['Power law error. Sum of probabilities equals ' ...
        num2str(sum(prop))])
    stop
end

% Total Number of floes
Nftot = round(c*pi*Rmiz^2/(pi*sum(prop.*(Rad.^2))));

% Number of floes per bin
Nf = round(prop*Nftot);

% Distorted ice concentration
c_dum = sum(pi*Nf.*Rad.^2)/(pi*Rmiz^2);

% Adjusted width to maintain specified concentration
Rmiz_adj = sqrt(c_dum/c)*Rmiz;

% Sanity check
if abs(sum(pi*Nf.*Rad.^2)/(pi*Rmiz_adj^2)-c) > 1e-4
    disp('ice concentration error')
    stop
end

function pos = circle_packing(Nf, Rad, Rmiz)

% [pos, c_ice] = circle_packing(Nf, Rad, Rmiz)
%
% Generation of a FSD in a circular domain from a sampled
% distribution. 
tic

% Number of unique floes
Nfu = length(Rad);

% FSD in circular domain (origin assumed at centre)
win = 0;
att_n = 0;
max_att = 100;
while win == 0 
    res = 0;
    att_n = att_n + 1
    
    % Start with floes of largest size
    % Initialise first floe (in polar coordinates)
    post(1:2) = [sqrt((Rmiz.^2*rand)) 2*pi*rand];
    
    % Test (if intersecting boundary)
    test = 0;
    while test == 0
        del = Rmiz - (post(1) + Rad(end));
        if del > 0
            pos = [post Nfu Rad(Nfu)];
            test = 1;
        else
            post(1:2) = [sqrt((Rmiz.^2*rand)) 2*pi*rand];
        end
    end
    
    % Subsequent floes
    for p = 2:Nf(end)
        post(1:2) = [sqrt((Rmiz.^2*rand)) 2*pi*rand];
        
        % Test (if intersects boundary or other floes)
        test = 0;
        while test == 0
            % boundary
            delb =   Rmiz - (post(1) + Rad(end));
            idelb = find(delb < 0);
            
            % Floes
            delc = post(1)^2 + pos(:,1).^2 - ...
                2*post(1)*pos(:,1).*cos(pos(:,2) - post(2)) - ...
                (pos(:,4) + Rad(end)).^2;
            idelc = find(delc<0);
            if isempty(idelb) && isempty(idelc)
                pos = [pos; post Nfu Rad(Nfu)];
                test = 1;
            else
                post(1:2) = [sqrt((Rmiz.^2*rand)) 2*pi*rand];
            end
        end
    end
    
    % Fill in the row in decreasing floe size order
    for s = Nfu-1:-1:1
        for p = 1:Nf(s)
            post(1:2) = [sqrt((Rmiz.^2*rand)) 2*pi*rand];
            
            % Test (if intersects boundary or other floes)
            test = 0;
            count = 0;
            while test == 0
                count = count+1;
                % boundary
                delb =   Rmiz - (post(1) + Rad(s));
                idelb = find(delb < 0);
                
                % Floes
                delc = post(1)^2 + pos(:,1).^2 - ...
                    2*post(1)*pos(:,1).*cos(pos(:,2) - post(2)) - ...
                    (pos(:,4) + Rad(s)).^2;
                idelc = find(delc<0);
                if isempty(idelb) && isempty(idelc)
                    pos = [pos; post s Rad(s)];
                    test = 1;
                else
                    post(1:2) = [sqrt((Rmiz.^2*rand)) 2*pi*rand];
                end
                
                if count > 100000
                    warning('Max number of iterations reached. Could not generated circular MIZ!')
                    res = 1;
                    break
                end
            end
            if res == 1
                break
            end
        end
        if res == 1
            break
        end
    end
    if res == 0
        win = 1;
    elseif att_n > max_att
        error('MIZ generation unsuccessful: maximum number of attempts reached!')
    end
end
pos = pos(:,1:3);
toc
disp('Circular MIZ successfully created!')

function visu_geom(pos,rads)

np=size(pos,1);
% X=zeros(51,np);
% Y=zeros(51,np);
figure('color','w')
% subplot(1,2,1)
for i=1:np
%     t=0;
%     for psi=0:2*pi/50:2*pi
%         t=t+1;
%         X(t,i)=pos(i,1)*cos(pos(i,2))+pos(i,3)*cos(psi);
%         Y(t,i)=pos(i,1)*sin(pos(i,2))+pos(i,3)*sin(psi);
%     end
%     hold on
%     plot(X(:,i),Y(:,i),'k')

    dat = [pos(i,1)*cos(pos(i,2)) - rads(pos(i,3)), ...
        pos(i,1)*sin(pos(i,2)) - rads(pos(i,3)), ...
        2*rads(pos(i,3)), 2*rads(pos(i,3))];
    rectangle('Position',dat,'Curvature',[1 1],'FaceColor',...
        [0.9 0.9 0.9],'EdgeColor','k')%,'LineWidth',1)
end
axis equal
axis off

