function [DTM_ext,DTM_int,DTM_damp,N] = Circular_Plate_Solver(h, Rad, nu, ...
    d, alpha, beta, k, kappa, M)

% [DTM_ext, DTM_int, DTM_damp, N] = Circular_Plate_Solver(h, Rad, nu, d, alpha, k, kappa, M)
%
% Solves the circular plate problem using eigenfunction matching. We 
% compute the interior and exterior diffraction transfer matrices, which 
% map the incident field (expressed in polar coordinates) to the interior 
% and exterior scattered fields, respectively. The Fourier modes decouple 
% so the DTMs are given for each n, -N \le n \le N, and have size M+1. A 
% symmetry argument may be used to show that the DTMs for n and -n are the
% same, therefore reducing the number of required evaluations.  
%
%%%%%%%% Outputs:
% - DTM_ext: exterior diffraction transfer matrices
% - DTM_int: interior diffraction transfer matrices
% - DTM_damp: damped travelling modes diffraction transfer matrices
%%%%%%%% Inputs:
% - h: water depth
% - Rad: floe radius
% - nu: Poisson's ratio
% - d: floe draught
% - alpha: frequency parameter
% - beta: vector of floe stiffness parameters
% - k: wavenumbers solutions of the free surface dispersion relation
% - kappa: wavenumbers solutions of the ice-covered dispersion relation
% - M: number of vertical modes (M(1) for single floe solution and M(2) for
% multiple scattering solution)
%
% Code written by Fabien Montiel and last modified on 04/04/23.


%% Vertical modes weights
WATER_WEIGHT = WeightVertDef(k, h);
ICE_WEIGHT = WeightVertDef(kappa, h - d);

%% Vertical modes derivatives at fluid/plate surface
Dxi = VertModeBasisDiff(-d, kappa, h)./ICE_WEIGHT;

%% Linear dependence vertical modes under plate
V = LinDepDampedModes(alpha, beta, d, kappa, ICE_WEIGHT, h, M(1));

%% Matrices of innerproducts 
% Note: All inner-product matrices are scaled by h/2

% Free-surface/free-surface:
D_0 = zeros(M(1)+1);
for p = 1:M(1)+1
    D_0(p,p) = innerproduct(k(p), k(p), h)/WATER_WEIGHT(p)^2*(2/h);
end

% Ice/ice:
D = zeros(M(1)+1);
for p = 1:M(1)+1
    for m = 1:M(1)+1
        D(p,m) = innerproduct(kappa(p+2), kappa(m+2), h - d)/...
            (ICE_WEIGHT(p+2)*ICE_WEIGHT(m+2))*(2/h);
    end
end

% Write dr_Phi_n = u_n(z) * U_n, with u_n(z) the vertical projection
% basis and U_n the vector of amplitudes. We can choose the basis to be any
% basis oon the domain [-h, -d]. Two choices are proposed here:

D_0g = zeros(M(1)+1);
for p = 1:M(1)+1
    for m = 1:M(1)+1
        D_0g(p,m) = innerproduct(k(p), kappa(m+2), h - d)/...
            (WATER_WEIGHT(p)*ICE_WEIGHT(m+2))*(2/h);
    end
end
D_g=D;


%% Angular modes decoupling - Diffraction transfer matrices

% Number of angular modes (Wiscombe formula)
N = ang_mode_wiscombe(k(1)*Rad);

% Init
DTM_ext  = cell(1,N+1);
DTM_int  = cell(1,N+1);
DTM_damp = cell(1,N+1); 

for n = 0:N
%% Free Edge conditions
    % Bending moment: Ln_1*Bn + Ltn_1*Btn = 0
    Ln_1 = Dxi(3:end).*((Rad*kappa(3:end)).^2 + (1-nu).*(...
        Rad*kappa(3:end).*dbesselj(n,kappa(3:end)*Rad,1)./...
        besselj(n,kappa(3:end)*Rad,1) - n^2));
    
    Ltn_1 = Dxi(1:2).*((Rad*kappa(1:2)).^2 + (1-nu).*(Rad*kappa(1:2).*...
        dbesselj(n,kappa(1:2)*Rad,1)./besselj(n,kappa(1:2)*Rad,1) - n^2));
    
    % Shear stress: Ln_2*Bn + Ltn_2*Btn = 0
    Ln_2 = Dxi(3:end).*((Rad*kappa(3:end)).^3.*...
        dbesselj(n,kappa(3:end)*Rad,1)./besselj(n,kappa(3:end)*Rad,1) + ...
        n^2*(1-nu).*(Rad*kappa(3:end).*dbesselj(n,kappa(3:end)*Rad,1)./...
        besselj(n,kappa(3:end)*Rad,1) - 1));
    
    Ltn_2 = Dxi(1:2).*((Rad*kappa(1:2)).^3.*...
        dbesselj(n,kappa(1:2)*Rad,1)./besselj(n,kappa(1:2)*Rad,1) + ...
        n^2*(1-nu).*(Rad*kappa(1:2).*dbesselj(n,kappa(1:2)*Rad,1)./...
        besselj(n,kappa(1:2)*Rad,1) - 1));
     
    % Damped modes scattering matices
    D_damp=-inv([Ltn_1;Ltn_2])*[Ln_1;Ln_2];

%% Matching conditions
    % Matching 1: D_0*DE_n_I*A_n_I + D_0*DE_n_S*A_n_S = D_0g*U_n
    DE_n_I = diag(k.*dbesselj(n,k*Rad,1)./besselj(n,k*Rad,1));
    DE_n_S = diag(k.*dbesselh(n,1,k*Rad,1)./besselh(n,1,k*Rad,1));
    
    % Matching 2: D*DT_n*B_n = D_g*U_n
    DT_n = diag(kappa(3:end).*dbesselj(n,kappa(3:end)*Rad,1)./...
        besselj(n,kappa(3:end)*Rad,1)) + ...
        V*diag(kappa(1:2).*dbesselj(n,kappa(1:2)*Rad,1)./...
        besselj(n,kappa(1:2)*Rad,1))*...
        D_damp;

    % Matching 3: D_0g.'(E_n_I*A_n_I + E_n_S*A_n_S) = D_g.'*T_n*B_n
    E_n_I = eye(M(1)+1);
    E_n_S = eye(M(1)+1);
    T_n = eye(M(1)+1) + V*D_damp;

    
%% Expressing scattered wave amplitudes
    % open water: A_n_S = M_n_SI*A_n_I + M_n_SU*U_n
    % plate cover: B_n  = M_n_BU*U_n
    M_n_SI = -(D_0*DE_n_S)\(D_0*DE_n_I);
    M_n_SU = (D_0*DE_n_S)\D_0g;
    M_n_BU = (D*DT_n)\D_g;
    
%% Potential normal derivative at plate's edge
    % U_n = M_n_UI*A_n_I
    M_n_UI = (D_g.'*T_n*M_n_BU-D_0g.'*E_n_S*M_n_SU)\(D_0g.'*...
        (E_n_I+E_n_S*M_n_SI));
    
%% Angularly decoupled diffraction transfer matrices
    % A_n_S = S_n_ext*A_n_I
    % B_n = S_n_int*A_n_I
    S_n_ext = M_n_SI+M_n_SU*M_n_UI;
    S_n_int = M_n_BU*M_n_UI;
    
%% Reduced diffraction transfer matrices (for interaction problem)
    DTM_ext{1,n+1} = S_n_ext(1:M(2)+1,1:M(2)+1);
    DTM_int{1,n+1} = S_n_int(1:M(2)+1,1:M(2)+1);
    DTM_damp{1,n+1} = D_damp(:,1:M(2)+1);

%% Rescaling (maps unscaled cylindrical expansion coefficients)
    DTM_ext{1,n+1} = diag(1./besselh(n,1,k(1:M(2)+1)*Rad))*DTM_ext{1,n+1}*...
        diag(besselj(n,k(1:M(2)+1)*Rad));
    DTM_int{1,n+1} = diag(1./besselj(n,kappa(3:M(2)+3)*Rad))*DTM_int{1,n+1}*...
        diag(besselj(n,k(1:M(2)+1)*Rad));
    DTM_damp{1,n+1} = diag(1./besselj(n,kappa(1:2)*Rad))*DTM_damp{1,n+1}*...
        diag(besselj(n,kappa(3:M(2)+3)*Rad));
end


function N = ang_mode_wiscombe(x)

% N = ang_mode_wiscombe(x)
%
% Calculates the number of angular modes necessary to resolve scattering by
% a single scatterer of radius a at a wavenumber k to high precision. 
% We have x=k*a.

if x <= 8
    N = x + 4*x^(1/3) + 1;
elseif x < 4200
    N = x + 4.05*x^(1/3) + 2;
else
    N = x + 4*x^(1/3) + 2;
end

N = ceil(N);