function [S_mat] = Scat_MIZ_single_slab_solver(FSD, FSP, Width, ...
    chi, Ntr, Nev, k, T0, Gamma, M, N)
    
% S_mat = Scat_MIZ_single_slab_solver(h, FSD, FSP, Width, d, beta, nu, alpha, Inter_map, chi, Ntr, Nev, k, M, N, P)
%
% Computes the scattering matrix of a slab containing a finite array of 
% floes, independently from the position of the slab. It maps an arbitrary 
% incident directional spectrum to the scattered directional spectra on 
% either side of the slab. 


% Number of floes
Np = size(FSD,1);

%% Multiple scattering problem - diffraction transfer matrix of array
T_scat = Multi_scat_DTM(Np, FSD, FSP, k, T0, Gamma, M, N);

%% Reflection and transmission kernel matrices 
tic
% Reflection and transmission kernel matrices (non-integrated)
[R_m_ker, T_p_ker, R_p_ker, T_m_ker] = ...
    scat_kernel(FSD, FSP(:,1), Width, k, T_scat, chi, Ntr, Nev, M, N);
toc
% Numerical integration map
%
% A_R(chi) = \int_{-\pi/2}^{\pi/2} R_m_ker(chi,gam) A_In(gam) d(gam)
%          = R_m_ker(chi,gam)*Mat_int*A_In(gam)
%
% A_T(chi) = \int_{-\pi/2}^{\pi/2} T_p_ker(chi,gam) A_In(gam) d(gam)
%          = T_p_ker(chi,gam)*Mat_int*A_In(gam)
%
tic
%%% Trapezoidal rule
del_tr = pi/(2*(Ntr-1));
if length(chi) == Ntr
    M_numer = del_tr*diag([1 2*ones(1,Ntr-2) 1]);
else
    del_ev = (-chi(1)-pi/2)/(2*Nev);
    M_numer = diag([del_ev 2*del_ev*ones(1,Nev-1) (del_ev+del_tr) ...
        2*del_tr*ones(1,Ntr-2) (del_ev+del_tr) ...
        2*del_ev*ones(1,Nev-1) del_ev]);
    M_numer_ev = diag([del_ev 2*del_ev*ones(1,2*Nev-1) del_ev]);
end

for m = 2:M+1
    M_numer = blkdiag(M_numer, M_numer_ev);
end
%%%

% Phase change at slab boundaries
Mat_PC = diag(exp(1i*k(1)*Width*cos(chi)));
chi_ev = imag([chi(1:Nev) 0 chi(Ntr+Nev+1:end)])*1i;
for m = 2:M+1
    Mat_PC = blkdiag(Mat_PC, diag(exp(1i*k(m)*Width*cos(chi_ev))));
end

% Reflection and transmission matrices
S_R_m  = R_m_ker*M_numer;
S_TT_m = T_m_ker*M_numer + Mat_PC;
S_TT_p = T_p_ker*M_numer + Mat_PC;
S_R_p  = R_p_ker*M_numer;

% Scattering matrix
S_mat = [S_R_m,  S_TT_m;...
     S_TT_p, S_R_p];

% % Mapping from directional spectra to local polar amplitude
% S_Car2Pol = [MapC2P_m*M_numer MapC2P_p*M_numer];
toc