function mroots = dispersion_free_surface(alpha,N,h)

% mroots = dispersion_free_surface(alpha,N,h)
%
% Routine to calculate the roots of the dispersion relation:
%
% K*tanh(K*h) = alpha,
%
% For simplicity, we assume that alpha is real. If the value for h is not 
% given, the default value is h = 1.


if floor(N)~=N || N<0
    error('N should be a positive integer')
end

if nargin == 2
    h = 1;
else
    alpha = h*alpha; % scaling for the solution 
end

% init
mroots = zeros(1,N+1);

% Real root
mroots(1) = realroot(alpha,alpha);

% Imaginary roots
for n = 1:N
    mroots(n+1) = 1i*imagroot(alpha,(2*n-1)*pi/2+1/alpha);
end

mroots = mroots/h;
  

function out = realroot(alpha,guess)
%calculates the root nearest the root guess using Newton's method.
ans1 = guess+1; 
out = guess;
while abs(ans1 - out) > 1e-9
    ans1 = out;
    out = ans1 - f_re(ans1,alpha)/difff_re(ans1);
end

function out = imagroot(alpha,guess)
%calculates the root nearest the root guess using Newton's method.
ans1 = guess+1; 
out = guess;
while abs(ans1 - out) > 1e-9
    ans1 = out;
    out = ans1 - f_im(ans1,alpha)/difff_im(ans1);
end

function out = f_re(z,alpha)
out = z*tanh(z) - alpha;

function out = difff_re(z)
out = tanh(z) + z*sech(z).^2;

function out = f_im(z,alpha)
out = z*tan(z) + alpha;

function out = difff_im(z)
out = tan(z) + z*sec(z).^2;